package agent;

import mailbox.AgentMailbox;
import mailbox.Message;

import java.awt.*;

import gameserver.RoboterInfo;

/**
 * Oberklasse fr alle Agenten. Jede konkrete Implementierung muss diese beerben,
 * um in der Simulation benutzbar zu sein. Die konkrete Vorgehensweise ist in
 * der zum Seminar angefertigten Ausarbeitung beschrieben.
 *
 *
 *
 * User: Heiner Frhling
 * Date: 26.10.2003
 * Time: 18:09:12
 *
 */
public abstract class Agent implements Runnable{

    /**
     * Der Name
     */
    protected String ivName;
    protected AgentMailbox ivAgentMailbox;
    protected boolean ivRunning = false;
    protected RoboterInfo ivRoboterInfo;
    protected boolean ivNeedsDialog = false;
    protected Frame ivMyDialog;




    /**
     * Default-Konstruktor fr Reflection-API Class.forName().newInstance() - Aufruf
     */
    public Agent(){}

    /**
     * Implementierung der im Inteface Runnable definierten Standardmethode fr die
     * Logik eines Threads
     */
    public void run(){
        if (ivRunning){
             try {
                Thread.sleep(50);
            } catch (InterruptedException e) {
                e.printStackTrace();  //To change body of catch statement use Options | File Templates.
            }

        }
    }

    public String[] getSensoren() {
        return ivRoboterInfo.getSensoren();
    }

   /**
    * Getter fr den Namen des Agenten.
    * @return Der Name des Agenten
    */
    public String getName() {
        return ivName;
    }

    /**
     * Setter fr den Namen des Agenten
     * @param ivName Der Name des Agenten
     */
    public void setName(String ivName) {
        this.ivName = ivName;
    }

    /**
     * Setter fr die Mailbox des Agenten
     * @param ivAgentMailbox Die dem Agenten zugewiesene Mailbox
     */
    public void setAgentMailbox(AgentMailbox ivAgentMailbox) {
        this.ivAgentMailbox = ivAgentMailbox;
    }


    /**
     * Setter fr die Variable ivRunning. Diese wird in der run()-Schleife des Agenten dazu verwendet, um die
     * Ausfhrung zu suspendieren.
     * @param ivRunning
     */
    public synchronized void setRunning(boolean ivRunning) {
        this.ivRunning = ivRunning;
    }

    /**
     * ber diese Methode lsst sich nachprfen, ob der Agent noch aktiv ist oder die Ausfhrung beendet wurde
     * @return Wahrheitswert, ob der Agent noch luft oder nicht.
     */
    public synchronized boolean isRunning() {
        return ivRunning;
    }

    /**
     * Gibt an, ob der Agent eine GUI bentigt. Diese Methode wird per Callback bei der
     * Initialisierung aufgerufen. Falls sie von der konkreten Implementierung nicht berschrieben
     * wird, wird false zurckgegeben.
     * @return Wahrheitswert, der angibt, ob der Agent einen Dialog braucht
     */
    public boolean needsDialog() {
        return ivNeedsDialog;
    }


    // Methoden zur Verwaltung von Dialogen
    public void setNeedsDialog(boolean ivNeedsDialog) {
        this.ivNeedsDialog = ivNeedsDialog;
    }

    //



    /**
     * Diese Defaultimplementierung returned einen Frame ohne Funktion und mu
     * daher von den erbenden Klassen berschrieben werden!!!
     * @return Den vom Agenten bentigten Frame
     */
    public Frame getFrame() {
        return ivMyDialog;
    }

    /**
     * Sendet eine Nachricht an den Server. In der Nachricht mu nur der Betreff
     * und gegebenenfalls der Inhalt angegeben werden, den Rest fgt die Mailbox
     * hinzu.
     *
     * Falls der Betreff fehlt, wird die Nachricht nicht gesendet.
     *
     * @param message Die zu sendende Nachricht
     */
    public void sendMessageToServer(Message message){
        // UserUnsinn abfangen!!!!
        // Mindestvoraussetzung fr die Verarbeitung auf dem Server ist ein Betreff
        try {
            message.getBetreff();
            ivAgentMailbox.sendMessageToServer(message);
        } catch (Exception e) {
            // sende keine Nachricht, falls getBetreff() eine NullpointerException wirft!!!
        }
    }


    /**
     * Sendet eine Broadcast-Nachricht an alle Agenten. In der Nachricht mu nur
     * der Betreff  und gegebenenfalls der Inhalt angegeben werden, den Rest fgt
     * die Mailbox hinzu.
     *
     * Falls der Betreff fehlt, wird die Nachricht nicht gesendet.
     *
     * @param message
     */
    public void sendBroadCastToAgents(Message message){
        // UserUnsinn abfangen!!!!
        // Mindestvoraussetzung fr die Verarbeitung auf dem Server ist ein Betreff
        try {
            message.getBetreff();
            ivAgentMailbox.sendBroadcastToAgents(message);
        } catch (Exception e) {
            // sende keine Nachricht, falls getBetreff() eine NullpointerException wirft!!!
        }
    }


    /**
     * Sendet eine Nachricht an den durch den Namen bestimmten Agenten. In der Nachricht mu nur
     * der Betreff  und gegebenenfalls der Inhalt angegeben werden, den Rest fgt
     * die Mailbox hinzu.
     *
     * Falls der Betreff fehlt, wird die Nachricht nicht gesendet.
     *
     * @param msg
     * @param adresse Name des Agenten
     */
    public void sendMessageToAgent(Message msg, String adresse){
        ivAgentMailbox.sendMessageToAgent(msg, adresse);
    }

    /**
     *
     * @return Message oder null, falls keine Nachricht vorhanden
     */
    public Message getNextMessage() {
        return ivAgentMailbox.getNextMessage();
    }

    /**
     * Gibt das Objekt mit den Informationen ber den angeschlossenen
     * Roboter zurck.
     * @return Objekt mit Informationen zum angeschlossenen Roboter
     */
    public RoboterInfo getRoboterInfo() {
        return ivRoboterInfo;
    }

    /**
     * Setter fr die Roboterinformationen
     * @param ivRoboterInfo
     */
    public void setRoboterInfo(RoboterInfo ivRoboterInfo) {
        this.ivRoboterInfo = ivRoboterInfo;
        // Konsolenausgaben zum Test
        log("setRoboterInfo");
        log("Antrieb:  " + ivRoboterInfo.getAntrieb());
        log("Energie:  " + ivRoboterInfo.getEnergieVorrat());
        log("Richtung: " + ivRoboterInfo.getRichtung());
        for(int  i = 0; i < ivRoboterInfo.getSensoren().length; i++){
            log("Sensor " + i + " : " + ivRoboterInfo.getSensoren()[i]);
        }



    }

    private void log(String msg){
        System.out.println(getClass().toString() + "  ->  " + msg);
    }
}
