package agent;

import java.io.File;
import java.util.Vector;

/**
 * Utility-Klasse zum automatischen Laden der Agenten
 *
 * User: Standard
 * Date: 03.11.2003
 * Time: 17:36:02
 *
 */
public class AgentLoader {


    // Implizite Annahme: derzeit befinden sich alle Agenten im Package impl
    // unterhalb von Agent
    // dies knnte ber eine Datenbank dynamischer / flexibler gestaltet werden.
    private String ivPackage;
    private String ivAgentImplPackage;


    public AgentLoader(){
        String lvPack = this.getClass().getPackage().toString();
        ivPackage = lvPack.substring(8); // Entfernen von "package "
        ivAgentImplPackage = ivPackage + ".impl";

    }


    /**
     * Holt die Namen der Agenten aus dem impl-Verzeichnis. Dabei wird berprft, ob
     * es sich bei den geladenen Strings um Instanzen der Superklasse Agent handelt.
     * @return Array mit den Nammen aller verfgbaren Agenten
     */
    public String[] loadAgentClassNamesValidate(){
        String[] lvDirectoryEntries = null;
        Vector agentnames = new Vector();

        File f = new File("src/" + ivAgentImplPackage.replace('.','/'));

        lvDirectoryEntries = f.list();

        for (int i = 0; i < lvDirectoryEntries.length; i++){
            // hole die Pos. des Punktes
            int index = lvDirectoryEntries[i].lastIndexOf(".");
            // Abschneiden des Datei-Suffixes
            lvDirectoryEntries[i] = lvDirectoryEntries[i].substring(0, index);

            // prfe, ob die Klasse von der Superklasse Agent abgeleitet ist!!
            String lvFullname = ivAgentImplPackage + "." + lvDirectoryEntries[i];

            try {
                Class c = Class.forName(lvFullname);
                System.out.println(c.toString());

                System.out.println("Kompatibilittstest kann " +c.toString() + " auf Agent gecastet werden? " + Agent.class.isAssignableFrom(c));
                if (Agent.class.isAssignableFrom(c)){
                    agentnames.add(lvDirectoryEntries[i]);
                }
            } catch (ClassNotFoundException e) {

            }

        }// ende for-schleife fr Durchlauf durch die geladenen Namen

        String[] lvReturnStrings =  new String[agentnames.size()];
        if(agentnames.size() > 0){
            for (int j = 0; j < agentnames.size(); j++){
                lvReturnStrings[j] = (String) agentnames.get(j);
            }
        }

        return lvReturnStrings;
    }

    /**
     * Diese Methode instantiiert zu einem bergebenen String-Array mit den Namen der
     * Agentenklassen die entsprechenden Objekte
     *
     * @param agentnames Agentennamen ohne Packageprefix, dieses wird in der Methode gesetzt
     * @return Array mit den erzeugten Agenten
     */
    public Agent[] getAgentObjects(String[] agentnames){
        Agent[] agentinstances = null;
        Vector lvInstances = new Vector();

        for (int i = 0; i < agentnames.length; i++ ){
            String lvFullname = ivAgentImplPackage + "." + agentnames[i];
            System.out.println(ivAgentImplPackage);

            try {
                Class c = Class.forName(lvFullname);
                System.out.println(c.toString());
                System.out.println("Ist die Klasse vom Typ Agent? " + c.getSuperclass().equals(Agent.class));
                if (c.getSuperclass().equals(Agent.class)){

                    try {
                        lvInstances.add(c.newInstance());
                        System.out.println("Klappt die Instantiierung? " + lvInstances.lastElement().toString());
                    } catch (InstantiationException e) {
                        // tue nichts
                    } catch (IllegalAccessException e) {
                        // tue nichts
                    }
                }
            } catch (ClassNotFoundException e) {
                // Tue nichts
            }
        }

        agentinstances = new Agent[lvInstances.size()];
        for (int i = 0; i < lvInstances.size(); i++){
            agentinstances[i] = (Agent) lvInstances.get(i);
        }

        return agentinstances;
    }


    /**
     * Erzeugt zum Nameneiner Agentenklasse das entsprechende Agentenobjekt.
     * Diese Methode prft NICHT nach, ob es sich um einen gltigen Namen handelt.
     * @param agentname Name der Klasse des zu initialisierenden Agenten
     * @return Der initialisiserte Agent
     */
    public Agent getAgentInstance(String agentname){
        Agent lvInstance = null;


        String lvFullname = ivAgentImplPackage + "." + agentname;


        try {
            Class c = Class.forName(lvFullname);
            // Testloggings
            System.out.println(c.toString());
            System.out.println("Ist die Klasse vom Typ Agent? " + c.getSuperclass().equals(Agent.class));
            if (Agent.class.isAssignableFrom(c)){
                try{
                lvInstance = (Agent) c.newInstance();
                    System.out.println("Klappt die Instantiierung? " + lvInstance.toString());
                } catch (InstantiationException e) {
                    // tue nichts
                } catch (IllegalAccessException e) {
                    // tue nichts
                }
            }
        } catch (ClassNotFoundException e) {
            // Tue nichts
        }

        return lvInstance;
    }


}
