package editor;


import agent.Agent;
import agent.AgentLoader;

import mailbox.MailboxSystem;
import mailbox.Message;
import gameserver.Gameserver;
import gameserver.Roboter;

import java.util.Vector;
import java.util.Enumeration;
import java.awt.*;



/**
 * Die Klasse fhrt die Steuerung des Simulationsablaufs durch.
 *
 * User: Standard
 * Date: 04.11.2003
 * Time: 00:26:49
 *
 */
public class SimulationController {

    private Gameserver ivGameserver;
    private Spielfeld ivSpielfeld;
    private MailboxSystem ivMailboxSystem;
    private AgentLoader ivAgentLoader;
    private Vector ivThreadHolder;
    private SimulationGUI ivSimulationGUI;

    // Enthlt nur Objekte vom Typ Agent
    private Vector ivAgentenHolder;
    private Vector ivAgentsNeedingFrameHolder;

    private Thread ivGameServerThread;



    /**
     * Default-Konstruktor
     */
    public SimulationController(){
        ivAgentLoader = new AgentLoader();
        ivMailboxSystem = new MailboxSystem();
        ivThreadHolder = new Vector();
        ivAgentenHolder = new Vector();
        ivAgentsNeedingFrameHolder = new Vector();
    }


    /**
     * Diese Methode fhrt die Initialisierung der gesamten Simulation durch.
     * Das Spielfeld wird vorher erzeugt und als Parameter bergeben. Es enthlt
     * alle notwendigen Daten, so da Agenten und das Mailboxsystem initialisiert werden
     * knnen.
     *
     * @param aSpielfeld Das fertige, aus der Datenbank geladene und initialisierte Spielfeld
     */
    public void initSimulation(Spielfeld aSpielfeld){
        ivSpielfeld = aSpielfeld;
        ivGameserver = new Gameserver(ivSpielfeld);
        // Dem Server den Controller anmelden fr kills
        ivGameserver.setController(this);
        // Hole die Agentenbezeichnung aus den Robbies und
        // fhre das Instantiieren durch, setze den Namen
        // Voraussetzung: beim Laden werden nur Ableitungen von Agent gesetzt
        // die auch gltig sind

        // Gameserver den Robbies bekanntmachen:
        Roboter[] lvRobbies = ivSpielfeld.getroboter();
                for (int i = 0; i < lvRobbies.length; i++){
                    lvRobbies[i].setServer(ivGameserver);
        }

        // Holen der Agenten!!!
        for (int i = 0; i < lvRobbies.length; i++) {
            log("Robotername: " + lvRobbies[i].getName());
            log("Agentenname: " + lvRobbies[i].getAgentenName());

            String lvAgentenKlassenName = lvRobbies[i].getAgentenName();
            Agent lvAgentInstance = ivAgentLoader.getAgentInstance(lvAgentenKlassenName);
            log("Agent instantiiert?  " + lvAgentInstance.toString());
            lvAgentInstance.setName(lvRobbies[i].getName());
            lvAgentInstance.setRoboterInfo(lvRobbies[i].getInformation());
            log("Braucht der Agent einen Dialog? " + lvAgentInstance.needsDialog());
            if (lvAgentInstance.needsDialog()){

                ivAgentsNeedingFrameHolder.add(lvAgentInstance);
                Frame lvAgentFrame =  lvAgentInstance.getFrame();
                lvAgentFrame.setVisible(false);
                log(lvAgentFrame.toString());

            }

            ivAgentenHolder.add(lvAgentInstance);
        }

        // Initialisiere das Mailbox-System
        // 1. Melde den Server an
        ivGameserver.setMailbox(ivMailboxSystem);
        // 2. Initialisiere das MailboxSystem.
        //    Dieses erzeugt sich seinen eigenen AgentMailbox-Vektor
        //    und meldet die Mailbox beim Agenten an
        Agent[] agents = new Agent[ivAgentenHolder.size()];
        // umkopieren in ein Array, (Agent[]) Cast klappt nicht...
        for (int i = 0; i < ivAgentenHolder.size(); i++){
            agents[i] = (Agent) ivAgentenHolder.get(i);
        }

        ivMailboxSystem.initMailboxSystem(agents);


        agents = null;

        // Initialisierung  des ThreadHolders fr die Agenten

        System.gc();

    }

    /**
     * Die Methode erzeugt fr jeden vorhandenen Agenten und den Server einen
     * Thread und startet zunchst den Server und dann die einzelnen Agenten-Threads
     */
    public void startSimulation(){
         ivMailboxSystem.setSystemRunning(true);
         ivGameServerThread = new Thread(ivGameserver);

         for (int i = 0; i < ivAgentenHolder.size(); i++){
            // Setzen der Variable
            ((Agent) ivAgentenHolder.get(i)).setRunning(true);


        }

        for (int i = 0; i < ivAgentenHolder.size(); i++){
            // Setzen der Variable
            //((Agent) ivAgentenHolder.get(i)).setRunning(true);
            // ThreadHolder als Vector, um einzelne zu entfernen (KILL!!!!!)
            ivThreadHolder.add( new Thread( ((Agent) ivAgentenHolder.get(i))));

        }


        // Zuerst den Server starten
        ivGameServerThread.start();
        // jeder einzelne Agententhread im Threadholder wird gestartet
        for (int i = 0; i < ivThreadHolder.size(); i++ ){
           ((Thread) ivThreadHolder.get(i)).start();
        }
        ivSimulationGUI.getMessageFrame().log(new Message("SIMCONTROLLER", "", Message.SYSTEMMESSAGE, "Simulation wurde gestartet"));
    }


    /**
     * Diese Methode ist dazu vorgesehen, die Simuation stoppen, um sie
     * ggf. wieder zu starten. Die Anwendung ist jedoch problematisch, da
     * bei Wiederanlauf die run()-Methode eines Agenten wieder von vorne
     * begonnen wird, also auch mit den einmaligen Initialisierungsschritten
     *
     */
    public synchronized void stopSimulation(){

        // Zuerst Agenten stoppen ...
        for (int i = 0; i < ivAgentenHolder.size(); i++ ){
           ((Agent) ivAgentenHolder.get(i)).setRunning(false);
        }

        ivMailboxSystem.setSystemRunning(false);
        // Vector neu initialisieren, damit die Threads auch wirklich gelscht werden
        ivThreadHolder = new Vector();
        // ... dann den Server
        ivGameserver.stopServer();
        ivSimulationGUI.getMessageFrame().log(new Message("SIMCONTROLLER", "", Message.SYSTEMMESSAGE, "Simulation wurde gestoppt"));
    }

    /**
     * Killt einen Agenten, seinen Thread und entfernt alle von ihm benutzten Elemente
     * @param agentName
     */
    public synchronized void killAgent(String agentName){
        log("killAgent("+agentName+")entered");
        for (int i = 0; i < ivAgentenHolder.size(); i++){
            log(((Agent) ivAgentenHolder.get(i)).getName());
            log("" + (((Agent) ivAgentenHolder.get(i)).getName().equals(agentName)));

            if (((Agent) ivAgentenHolder.get(i)).getName().equals(agentName)){
                // implizite Annahme aus der Initialisierung: Thread hat gleichen Index wie Thread (vgl. init)
                Thread threadToKill = (Thread) ivThreadHolder.remove(i);
                threadToKill = null;
                // killen des entsprechenden Dialogs!!!
                ivSimulationGUI.killAgentFrame(agentName);

                ((Agent) ivAgentenHolder.get(i)).setRunning(false);
                Agent agentToKill = (Agent) ivAgentenHolder.remove(i);
                agentToKill.setAgentMailbox(null);
                // Lschen der Agentenmailbox
                ivMailboxSystem.removeAgentMailbox(agentToKill.getName());
                ivSimulationGUI.getMessageFrame().log(new Message("SIMCONTROLLER","" ,Message.SYSTEMMESSAGE, "************************ Agent " + agentToKill.getName() + " ist auf eine Mine gelaufen! **************************************"));
                agentToKill = null;

                // expliziter Aufruf des GarbageCollectors
                System.gc();

            }// end if
        } // end for

        Enumeration e = ivAgentenHolder.elements();
        log("Verbleibende Elemente:");
        while (e.hasMoreElements()){
            log(e.nextElement().toString());
        }
    }



    private void log(String message){
        System.out.println(this.getClass().toString() + " -> " +message);
    }


    /**
     * bergibt alle Agenten, die einen Dialog bentigen, an die SimulationGUI,
     * damit diese fr jeden einen Button zum Ein- und Ausblenden anlegen kann.
     * @return
     */
    public Vector getAgentsNeedingDialog() {
        return ivAgentsNeedingFrameHolder;
    }


    /**
     * Diese Methode wird bentigt, um dem Mailboxsystem den MessageFrame der
     * SimulationGUI bekannt zu machen, ber den die ausgetauschten Nachrichten
     * gelogt werden knnen.
     *
     * @param ivSimulationGUI Die GUI, die die Simulation steuert.
     */
    public void setSimulationGUI(SimulationGUI ivSimulationGUI) {
        this.ivSimulationGUI = ivSimulationGUI;
        ivMailboxSystem.setMessageFrame(ivSimulationGUI.getMessageFrame());
    }


}
