package gameserver;

import mailbox.Message;
import mailbox.ServerMailbox;
import mailbox.ScanResult;
import editor.*;

/*
 * Created on 10.10.2003
 */

/**
 * Gameserver fr die Simulation der Minensuche.
 * Der Server befindet sich immer in einer der Phasen GSINIT, GSEVALUATING, GSEVALUATED oder GSSTARTED.
 * In der Init-Phase wird immer zunchst ein Spielfeld
 * @author Boris
 *
 */
public class Gameserver implements Runnable {
	final private static int FORWARD = 1;
	final private static int BACKWARD = -1;
	final private static String MYNAME = "Server";

	private Spielfeld spielfeld;
	private ServerMailbox mailbox;
	private boolean running;
	private SimulationController controller;


	/**Erzeugt einen neuen Gameserver
         *@param s das Spielfeld, welches der Server verwenden soll
         */
        public Gameserver(Spielfeld s) {
		spielfeld = s;
	}

	/**Stoppt die Ausf&uuml;hrung des Servers
         */
        public void stopServer() {
		this.running = false;
	}

	/**Gibt die Mailbox des Servers zur&uuml;ck.
         *@return die Mailbox des Servers
         */
        public ServerMailbox getMailbox() {
		return mailbox;
	}

	/**Weist dem Server eine Mailbox zu
         *@param mailbox die zuzuweisende Mailbox
         */
        public void setMailbox(ServerMailbox mailbox) {
		this.mailbox = mailbox;
	}

        /**Diese Methode war urspr&uuml;nglich dazu gedacht, 
         *Kommunikationspartner &uuml;ber den Start des Servers zu informieren,
         *tut aber implements jetzigen Zustand nichts mehr.
         *@deprecated
         */
        private void sendStartMessage() {
		//broadcast StartMessage an alle, die es wissen wollen...
	}

	/* private Roboter getRoboterByName(String name) throws RoboterUnknownException {
	   Enumeration enum = angemeldeteRoboter.elements();
	   Roboter robby;
	   while (enum.hasMoreElements()) {
	     robby = (Roboter) enum.nextElement();
	     if (name.equals(robby.getName())) {
	       return robby;
	     }
	   }
	   throw (new RoboterUnknownException(
	       "Es existiert kein Roboter mit diesem Namen!"));
	 }*/

	/**Die Hauptschleife des Servers
	 */
	public void run() {
		running = true;//damit er weiter luft
		Message message;
		Roboter robby;
		while (running) {//Solange er nicht angehalten wird
			do {
				message = mailbox.getNextServerMessage();//Nachricht aus Mailbox holen
				try {
					Thread.sleep(20);//Warten, um nicht zuviel Rechenzeit zu fressen
				} catch (InterruptedException e) {
				}
			} while (message == null);
			/*log(
				" Absender: "
					+ message.getAbsender()
					+ " - Betreff: "
					+ message.getBetreff());*/
			robby = spielfeld.getRoboterByName(message.getAbsender());//Zum Agenten gehrigen Roboter finden
			if (robby != null) {//Das sollte eigentlich immer true sein
				switch (message.getBetreff()) {//Betreffe dekodieren
					case Message.TURNNORTH :
						robby.turn(0, -1);
						break;
					case Message.TURNNORTHEAST :
						robby.turn(1, -1);
						break;
					case Message.TURNEAST :
						robby.turn(1, 0);
						break;
					case Message.TURNSOUTHEAST :
						robby.turn(1, 1);
						break;
					case Message.TURNSOUTH :
						robby.turn(0, 1);
						break;
					case Message.TURNSOUTHWEST :
						robby.turn(-1, 1);
						break;
					case Message.TURNWEST :
						robby.turn(-1, 0);
						break;
					case Message.TURNNORTHWEST :
						robby.turn(-1, -1);
						break;
					case Message.MOVEFORWARD :
						robby.move(FORWARD);
						break;
					case Message.MOVEBACKWARD :
						robby.move(BACKWARD);
						break;
					case Message.SCAN :
						{
							if (message.getContent() != null)//Sensorname angegeben?
								robby.doScan((String) message.getContent());
							break;
						}

					case Message.PING :
						{
							mailbox.sendMessageToAgent(
								message.getAbsender(),
								new Message(Message.PING));
							break;
						}
					case Message.GETINFO :
						{
							mailbox.sendMessageToAgent(
								message.getAbsender(),
								new Message(
									MYNAME,
									Message.INFO,
									robby.getInformation()));
						}
					case Message.REMOVEMINE:{
						robby.removeMine();
						break;
					}

					default ://Falls mal was unsinniges kommt...
						mailbox.sendMessageToAgent(
							message.getAbsender(),
							new Message(
								MYNAME,
								Message.ERR_NOTSUPPORTED,
								new Integer(message.getBetreff())));
				}
				//end switch
			} else {
				//Da hat wohl jemand keinen Roboter...
			}
		}
	}

	void sendMoveFinishedMessage(String empfaenger) {
		mailbox.sendMessageToAgent(
			empfaenger,
			new Message(MYNAME, Message.MOVEFINISHED, null));
	}

	void sendMineRemovedMessage(String empfaenger) {
		mailbox.sendMessageToAgent(
			empfaenger,
			new Message(MYNAME, Message.MINEREMOVED, null));
	}
	
	
	/*
        private void log(String message) {
		System.out.println(this.getClass().toString() + " -> " + message);
	}
        */
	/**Weist dem Server einen SimulationController zu
	 * @param controller der SimulationController
	 */
	public void setController(SimulationController controller) {
		this.controller = controller;
	}
	
        /**t&ouml; einen Agenten
         *@param name der Name des zu t&ouml;tenden Agenten
         */
	public void killRobot(String name) {
		controller.killAgent(name);
	}

	/**Sendet das Ergebnis eines Scans an einen Agenten
         *@param empfaenger der Empf&auml;nger der Nachrich
         *@param sensor der Name des Sensors
         *@param result das Ergebnisobjekt
         */
        public void sendScanResult(String empfaenger, String sensor, String[][] result) {
		ScanResult scanResult = new ScanResult(sensor, result);
		mailbox.sendMessageToAgent(
			empfaenger,
			new Message(MYNAME, Message.SCANRESULT, scanResult)
		);
		System.out.println("Server sendet Nachricht");
	}
	
	/**Sendet eine Warnung, dass der Roboter sich im Moment bewegt.
         *@param empfaenger der Name des zu warnenden Agenten
         */
        public void sendAlreadyMovingMessage(String empfaenger){
		mailbox.sendMessageToAgent(
			empfaenger,
			new Message(
			MYNAME,
			Message.ERR_ALREADYMOVING,
			null));
	}
	
        /**Sendet eine Warnung, dass der Roboter im Moment scannt.
         *@param empfaenger der Name des zu warnenden Agenten
         */
	public void sendAlreadyScanningMessage(String empfaenger){
		mailbox.sendMessageToAgent(
		empfaenger,
		new Message(
		MYNAME,
		Message.ERR_ALREADYSCANNING,
		null));
	}
	
        /**Sendet eine Warnung, dass der Roboter im Moment eine Mine entfernt.
         *@param empfaenger der Name des zu warnenden Agenten
         */
	public void sendAlreadyRemovingMessage(String empfaenger){
		mailbox.sendMessageToAgent(
		empfaenger,
		new Message(
			MYNAME,
			Message.ERR_ALREADYREMOVING,
			null));
	}


} // Ende Class
