package gameserver;
public class Matrix {
  private int[][] data;
  private int width;
  private int height;
  private static final int NULLWERT = 0;

  public Matrix(int width, int height) {
    data        = new int[width][height];
	
	if (width % 2 == 0) width++;
	if (height % 2 == 0) height++;

    this.width  = width;//Nur Matrizen zulassen, in denen der Robbi in der Mitte steht;
    this.height = height;

    for (int i = 0; i < width; i++) {
      for (int j = 0; j < height; j++) {
        data[i][j] = NULLWERT;
      }
    }
  }

  public void set(int x, int y, int value) { //IllegalCoordinatesException?
 	data[x][y] = value;
  }

  public int getWidth() {
    return width;
  }

  public int getHeight() {
    return height;
  }

  /************************************************************
  ** Einfacher erster Ansatz:                                **
  ** fr jeden Punkt der Zielmatrix wird der entsprechende   **
  ** Punkt der Ausgangsmatrix bestimmt. Hat den Vorteil,     **
  ** dass jeder Punkt definiert ist, aber auch den Nachteil, **
  ** dass vorhandene Punkte der Quellmatrix evtl. nicht er-  **
  ** scheinen.                                               **
  ************************************************************/
  public Matrix getRotatedMatrix(double angle) {
    int size        =(int) Math.ceil(Math.sqrt(width * width + height * height)) + 10;//Diagonale, damit whrend der Berechnung keine Bereichsprobleme auftauchen
    if (size % 2 == 0) size++;
    float middleX   = (width / 2); //ndern, falls auch gerade Ausdehnungen zugelassen sind
    float middleY   = (height / 2);
    float newMiddle = (size / 2);
    angle           = -angle; //Eigentlich rotiere ich die neue Matrix zurck, um die Punkte zu bestimmen.
    Matrix rotatedMatrix  = new Matrix(size, size);

    for (int rawX = 0; rawX < size; rawX++) {
      for (int rawY = 0; rawY < size; rawY++){
      	double x = rawX - newMiddle;
      	double y = rawY - newMiddle;
        //Die Drehformeln erhlt man durch die Additionstheoreme:
        double sourceX = x * Math.cos(angle) - y * Math.sin(angle) + middleX;//Quellkoordinaten berechnen
        double sourceY = y * Math.cos(angle) + x * Math.sin(angle) + middleY;
        if ((sourceX >= 0) && (sourceX <= width - 1) && (sourceY >= 0) && (sourceY <= height - 1)) { //Check, ob in Quellmatrix definiert
          rotatedMatrix.set(rawX, rawY, fetchValue(sourceX, sourceY));//Dann kopieren
          //if (data[sourceX][sourceY] != 0) System.out.println("" + data[sourceX][sourceY] + ", " + sourceX + ", " +sourceY);
        }
        else {
          //System.out.println("" + rawX + ", " + rawY + ", " + size);
          rotatedMatrix.set(rawX, rawY, NULLWERT); // sonst nix eintragen
        }
      }
    }
    rotatedMatrix = rotatedMatrix.trim();
    return rotatedMatrix;//fertich!!!
  }

  public int fetchValue(double x, double y) {
  	int[] pointsX = new int[2];
  	double[] weightsX = new double[2];
  	int[] pointsY = new int[2];
  	double[] weightsY = new double[2];
  	pointsX[0] = (int) Math.floor(x);
  	pointsX[1] = (int) Math.ceil(x);
	//if (pointsX[0] <  0) pointsX[0] = 0;
	//if (pointsX[1] > data.length) pointsX[1] = data.length; 
  	weightsX[0] = pointsX[1] - x;
  	weightsX[1] = 1 - weightsX[0];
	pointsY[0] = (int) Math.floor(y);
	pointsY[1] = (int) Math.ceil(y);
	weightsY[0] = pointsY[1] - y;
	weightsY[1] = 1 - weightsY[0];
	double ret = 0;
	for(int i = 0; i < 2; i++)
		for(int j = 0; j < 2; j++)
			try {ret += data[pointsX[i]][pointsY[j]] * weightsX[i] * weightsY[j];}
			catch (Exception e) {
				System.out.println(pointsX[0] + ", " + pointsX[1] + ", " + pointsY[0] + ", " + pointsY[1]);
				System.out.println(x + ", " + y); 
			}
  	return (int) Math.round(ret);
  }

  public String toString() {
    String returnString = "";
    for (int i = 0; i < height; i++) {
      for (int j = 0; j < width; j++) {
        returnString += "" + data[j][i] ;//+ "\t"
      }
      returnString += "\n";
    }
    return returnString;
  }
  
  private Matrix trim() {
    boolean finished = false;
    int xCut = 0;
    int yCut = 0;
    Matrix trimmedMatrix;
    
    
    for (int x = 0; (x <= width / 2) && !finished; x++) {
      for(int y = 0; (y < height) && !finished; y++) {
        if (data[x][y] != NULLWERT) finished = true;
        if (data[width - x - 1][y] != NULLWERT) finished = true;
      }
      if (finished) xCut = x;//+ 1 - 1
    }
    
    finished = false;
    
    for (int y = 0; (y <= height / 2) && !finished; y++) {
      for(int x = 0; (x < width) && !finished; x++) {
        if (data[x][y] != NULLWERT) finished = true;
        if (data[x][height - y - 1] != NULLWERT) finished = true;
      }
      if (finished) yCut = y;
    }
    //System.out.println("" + xCut + ", " + yCut);
    
    trimmedMatrix = new Matrix(width - 2 * xCut, height - 2 * yCut);
    for (int x = 0; x < trimmedMatrix.getWidth(); x++) {
      for (int y = 0; y < trimmedMatrix.getHeight(); y++) {
        trimmedMatrix.set(x, y, data[x + xCut][y + yCut]);
      }
    }
  	return trimmedMatrix;
  }
}