/*
 * Created on 25.10.2003
 */
package gameserver;

import editor.Gelaende;
import editor.Mine;
import editor.RoboterTyp;
import editor.SensorTyp;
import editor.Spielfeld;

import java.awt.Image;

/**
 * @author beumers
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/**Roboterinstanz. Enthlt genaue Position, Antrieb und andere
 * individuelle Eigenschaften.
 * Wird vom Server erstellt und verwaltet.
 * @author Boris
 */
public class Roboter {
	// public static String NORD = "Nord";
	// public static String NORDOST = "Nordost";
	//  public stat

	private int x; //reale Position des Roboters
	private int y; //
	private String name; //Name des Roboters
	private Antrieb antrieb; //ein Fahrwerk pro Roboter
	private RoboterTyp typ; //Typ des Roboters
	private boolean moving = false;
	private boolean removing = false;
	private boolean scanning = false;
	private Spielfeld spielfeld; //Spielfeld, auf dem sich der Robby befindet
	private int directionX; //soll im Intervall [-1,1] liegen: Easy to use
	private int directionY; //ebenso
	private Sensor[] sensoren;
	private Gameserver server;
	private long energievorrat;
	private String agentenName;
	/**Erstellt eine neue Roboterinstanz
	 * @param typ Der Typ des Roboters
	 * @param name Der Name des Roboter.
	 */
	public Roboter(RoboterTyp typ, String name, Spielfeld spielfeld) {
		this.typ = typ;
		this.name = new String(name);
		this.spielfeld = spielfeld;
		this.energievorrat = typ.getenergievorrat();
		//Sensoren anbringen:
		SensorTyp[] sensorTypen = typ.getSensorTypen(); //"Anleitung" laden
		sensoren = new Sensor[sensorTypen.length]; //"Slots" erstellen
		for (int i = 0; i < sensorTypen.length; i++) {
			sensoren[i] = new Sensor(sensorTypen[i], this);
			//Sensor bauen und anbringen
		}
		//Antrieb anbringen
		antrieb = new Antrieb(typ.getAntriebsTyp());
	}

	/**Weist dem Roboter ein Spielfeld zu, auf dem er sich befindet
	 * @param spielfeld Das Spielfeld, auf welches der Roboter gesetzt wird
	 */
	void setSpielfeld(Spielfeld spielfeld) {
		this.spielfeld = spielfeld;
	}

	/**Gibt einen String zurck, der die Blickrichtung des Roboters bezeichnet
	 * @return Eine String-Reprsentation der Blickrichtung (Nord, Nordost...)
	 */
	public String getBlickRichtung() {
		switch (directionX) {//x-Komponente checken
			case (-1) :
				switch (directionY) {//Y-Komponente checken
					case (-1) :
						return "Nordwest";
					case (0) :
						return "West";
					case (1) :
						return "Sdwest";
				}
			case (0) :
				switch (directionY) {//Y-Komponente checken
					case (-1) :
						return "Nord";
					case (1) :
						return "Sd";
				}
			case (1) :
				switch (directionY) {//Y-Komponente checken
					case (-1) :
						return "Nordost";
					case (0) :
						return "Ost";
					case (1) :
						return "Sdost";
				}
		}
		return "ILLEGAL!";//Fehler, sollte nicht vorkommen
	}

	
	/**Dreht den Roboter in die angegebene Richtung
	 * @param newDirX von -1 bis 1: Die neue X-Komponente der Richtung
	 * @param newDirY von -1 bis 1: Die neue Y-Komponente der Richtung
	 */
	public void turn(int newDirX, int newDirY) {
		if (energievorrat <= 0) //Ohne Energie nicht drehen
			return;
		if (moving) {//Whrend Bewegung nicht drehen
			sendAlreadyMovingMessage();//Warnen
			return;
		}
		if (removing) {//Whrend Minenentschrfung nicht drehen
			sendAlreadyRemovingMessage();//Warnen
			return;
		}

		for (int i = 0; i < sensoren.length; i++) {//Alle Sensoren auf Scanvorgang checken 
			if (sensoren[i].isScanning()) {//Wenn ja
				sendAlreadyScanningMessage();//Warnen...
				return;//...und nicht drehen
			}
		}

		if ((Math.abs(newDirX) > 1) || (Math.abs(newDirY) > 1))//Falls Richtung illegal...
			return; //...nichts tun (gibt's eigentlich nicht, da Server nur korrekte Richtungen schickt)
		directionX = newDirX; //X- 
		directionY = newDirY; //und Y-Komponente der Richtung setzen
		spielfeld.repaint();//  Spielfeld benachrichtigen, da nach turn ein repaint erfolgen soll
	}

	private long calcMoveEnergy(
		int toX,
		int toY) { //gibt bentigte Energie Energie in Millijoule zurck;
		final double g = 9.81; //Gravitationskonstante in m/s

		Gelaende g1 = spielfeld.getgelaende(x, y);//von Feld g1..
		Gelaende g2 = spielfeld.getgelaende(toX, toY);//...nach Feld g2..
		int deltaH = spielfeld.gethoehe(toX, toY) - spielfeld.gethoehe(x, y);//...ergibt sich dieser Hhenunterschied
		//Roboter ist im Busch, nicht auf dem Busch
		//deltaH in cm
		long m = getGesamtGewicht(); //Gesamtmasse m in Gramm
		long energy =
			Math.round(
				m * g * deltaH //Energie zur berbrckung der Hhe (+/-)
					+ typ.getAntriebsTyp().getenergieverbrauch(g2)); //+Verbrauch des Antriebs in diesem Gelnde
		return energy > 0 ? energy : 0; //Keine Energierckgewinnung mglich
	}

	/**Gibt die Bewegungsdauer fr die Bewegung auf ein bestimmtes benachbartes Feld zurck
	 * @param toX Die X-Koordinate des Feldes
	 * @param toY Die Y-Koordinate des Feldes
	 * @return Die Bewegungsdauer
	 * @throws ArithmeticException falls Geschwindigkeit = 0;
	 */
	private int calcTime(int toX, int toY) throws ArithmeticException {
		//v in m/min
		return 60000 / antrieb.getSpeed(spielfeld.getgelaende(toX, toY));//Geschwindigkeit in Bewegngsdauer umrechnen
	}

	/**
	 * @param direction 1 = vorwrts, -1 = Rckwrts
	 * @throws AlreadyMovingException, falls bereits Bewegung
	 */
	synchronized void move(int direction) {
		if (energievorrat <= 0)//Ohne Energie nicht bewegen
			return;
		if (moving) {//Wenn schon in Bewegung, nicht bewegen
			sendAlreadyMovingMessage();//Warnen
			return;
		}
		if (removing) {//Falls gerade Mine entfernt wird, nicht bewegen
			sendAlreadyRemovingMessage();//warnen
			return;
		}

		for (int i = 0; i < sensoren.length; i++) {//Alle Sensoren auf Scan untersuchen
			if (sensoren[i].isScanning()) {//Falls Sensor i scannt, nicht bewegen
				sendAlreadyScanningMessage();//warnen
				return;
			}
		}
		if (Math.abs(direction) != 1) {//Abbruch, falls ungltige Bewegungsrichtung
			return;
		}

		final int toX = x + directionX * direction;//Zielfeld errechnen
		final int toY = y + directionY * direction;

		if (!((toX >= 0) //Zielfeld nicht im Spielfeld?
			&& (toY >= 0)
			&& (toX < spielfeld.getBreite())
			&& (toY < spielfeld.getLaenge()))) {
			return;//Dann Abbruch
		}

		moving = true; //weitere Aktionen sperren 
		final Roboter robby = this; //this speichern, da aus Thread nicht erreichbar
		new Thread() {//Anonymen Thread erzeugen
			public void run() {//Dies wird ausgefhrt:
				try {
					Thread.sleep(calcTime(toX, toY));//Warten, bis Bewegung fertig
				} catch (InterruptedException e) {
				}
				energievorrat -= calcMoveEnergy(toX, toY);//Verbrauchte Energie abziehen
				synchronized (robby.getClass()) {//Das darf nur ein Roboter gleichzeitig tun
					if ((antrieb
						.getPassierbarkeit(spielfeld.getgelaende(toX, toY)))//Kann der Roboter berhaupt auf das Feld?
						&& !spielfeld.isRoboterAt(toX, toY)) {//Ist das Feld auch gerade frei?
						Mine mine = spielfeld.getmine(toX, toY);//Mine auf dem Feld?
						if (mine != null) {
							server.killRobot(robby.name);//Roboter tot
							spielfeld.deleteMine(toX, toY);//Mine weg
							spielfeld.deleteRoboter(robby);//Und Informationen aus dem Spielfeld entfernen
							return;
						}
						x = toX;
						y = toY;
					} else {
						energievorrat += calcMoveEnergy(toX, toY) / 2; //Falls nicht frei, Rckerstattung der halben Energie
					}
				}
				moving = false;//Wieder freigeben fr Aktionen
				server.sendMoveFinishedMessage(name);//Agent Bescheid geben
				spielfeld.repaint();//Spielfeld neu zeichnen
			}
		}
		.start();//Thread sofort starten

	}

	long getGesamtGewicht() {//Gewicht berechnen
		return typ.getgewicht()//Grundgewicht
			+ typ.getSensorenGesamtGewicht()//+Gewicht aller Sensoren
			+ antrieb.getGewicht();//+Gewicht des Antriebs
	}

	/**Gibt den Namen des Roboters als String zurck.
	 * @return Ein String mit dem Namen des Roboters.
	 */
	public String getName() {
		return new String(name);
	}

	/**Gibt die X-Komponente der Position des Roboters zurck
	 * @return die X-Komponente der Position
	 */
	public int getX() {
		return x;
	}

	/**Gibt die Y-Komponente der Position des Roboters zurck
	 * @return die Y-Komponente der Position
	 */
	public int getY() {
		return y;
	}

	/**Gibt das Spielfeld zurck, auf dem sich der Roboter befindet
	 * @return das Spielfeld zurck, auf dem sich der Roboter befindet
	 */
	Spielfeld getSpielfeld() {
		return spielfeld;
	}


	/**Gibt den Typnamen des Roboters zurck 
	 * @return Der Typname des Roboters
	 */
	public String gettypname() {
		return typ.getname();
	}

	/**Setzt den Roboter in die bezeichnete Spielfeldspalte
	 * @param x Die neue X-Position des Roboters
	 */
	public void setX(int x) {
		this.x = x;
	}

	/**Setzt den Roboter in die bezeichnete Spielfeldreihe
	 * @param y Die neue Y-Position des Roboters
	 */
	public void setY(int y) {
		this.y = y;
	}

	/**Gibt ein darstellbares Symbol des Roboters zurck
	 * @return Ein Bild des Roboters
	 */
	public Image getbild() {
		return typ.getbild();
	}

	/* Erzeugt eine Stringreprsentation des Roboters
	 * @return Eine String-reprsentation des Roboters
	 */
	public String toString() {
		return "name:"
			+ name
			+ "\n"
			+ "Position: ("
			+ x
			+ ", "
			+ y
			+ ")\n"
			+ " Antrieb: "
			+ antrieb.getName();

	}

	
	/**Gibt den Namen des zum Roboter gehrigen Agenten zurck
	 * @return Der Name des Agenten
	 */
	public String getAgentenName() {
		return agentenName;
	}

	/**Weist dem Roboter einen bestimmten Agenten zu
	 * @param agentenName Der Name des Agenten
	 */
	public void setAgentenName(String agentenName) {
		this.agentenName = agentenName;
	}
	
	/**Gibt die Typen der am Roboter befestigten Sensoren in einem Array zurck
	 * @return Ein Array mit den Namen aller am Roboter befestigten Sensoren 
	 */
	public String[] getSensorNames() {
		String[] ret = new String[sensoren.length];
		for (int i = 0; i < sensoren.length; i++) {
			ret[i] = sensoren[i].getTypName();
		}
		return ret;
	}
	
	/** Fhrt einen Scan mit einem Sensor durch
	 * @param sensorName Der Sensor, der scannen soll
	 */
	public void doScan(String sensorName) {
		if (energievorrat <= 0) //Ohne Energie kein Scan
			return;
		if (moving) {//Whrend der Bewegung kein Scan
			sendAlreadyMovingMessage();//warnen
			return;
		}
		if (removing) {//Wenn bereits Mine entschrft wird, kein scan
			sendAlreadyRemovingMessage();//warnen
			return;
		}

		for (int i = 0; i < sensoren.length; i++) {//Scannt einer der Sensoren bereits?
			if (sensoren[i].isScanning()) {
				sendAlreadyScanningMessage();//Dann nicht scannen
				return;
			}
		}

		//String[][] urls = new String[21][21];
		for (int i = 0; i < sensoren.length; i++) {
			if (sensorName.equals(sensoren[i].getTypName())) {//Richtigen Sensor aussuchen
				sensoren[i].doScan();//Und Scan beginnen
			}
		}
		return;
	}

	/**Dem Roboter einen Server zuweisen
	 * @param gameserver Der zugewiesene Server. ber diesen werden Nachrichten an die Agenten verschickt
	 */
	public void setServer(Gameserver gameserver) {
		server = gameserver;
	}

	/**Schickt dem Agenten ein Objekt, welches Informationen ber den Roboter enthlt
	 * @return Ein Objekt, welches Informationen ber Energievorrat, Sensoren, Antrieb und Blickrichtung enthlt
	 */
	public RoboterInfo getInformation() {
		String[] sensorNamen = new String[sensoren.length];
		for (int i = 0; i < sensoren.length; i++) {
			sensorNamen[i] = sensoren[i].getTypName();
		}
		return new RoboterInfo(
			typ.getenergievorrat(),
			energievorrat,
			sensorNamen,
			antrieb.getName(),
			getBlickRichtung());
	}


	/**Verringert die Energie des Roboters
	 * @param deltaE Der Energiebetrag, welcher abgezogen wird
	 */
	void decreaseEnergievorrat(long deltaE) {
		energievorrat -= deltaE;
	}

	public void sendScanResult(String sensorName, String[][] result) {
		server.sendScanResult(name, sensorName, result);
	}

	/**Entschrft eine Mine, die sich in Blickrichtung befindet
	 *
	 */
	public synchronized void removeMine() {
		if (energievorrat <= 0) //Ohne Energie geht's nicht
			return;
		if (moving) {//Nicht in der Bewegung
			sendAlreadyMovingMessage();//warnen
			return;
		}
		if (removing) {//Nicht, wenn schon entschrft wird
			sendAlreadyRemovingMessage();//warnen
			return;
		}

		for (int i = 0; i < sensoren.length; i++) {//Nicht whrend eines Scanvorgangs
			if (sensoren[i].isScanning()) {
				sendAlreadyScanningMessage();//warnen
				return;
			}
		}

		removing = true;//Roboter fr weitere Aktionen sperren
		new Thread() {//Anonymer Thread wie bei Bewegung
			public void run() {
				try {
					Thread.sleep(10000);//Entfernen dauert pauschal 10 Sekunden
				} catch (InterruptedException e) {
				}
				spielfeld.deleteMine(x + directionX, y + directionY); //Mine vom Spielfeld entfernen
				energievorrat -= 500;//Entfernen kostet konstanten Energiebetrag
				removing = false;//Roboter wieder freigeben
				server.sendMineRemovedMessage(name);//Agent Bescheid geben, das Roboter wieder frei ist
			}
		}
		.start();//Thread sofort starten

	}

	/**Sagt dem Server, dass der dem Agenten sagen soll, dass der Roboter momentan scant
	 * 
	 */
	void sendAlreadyScanningMessage() {
		server.sendAlreadyScanningMessage(name);
	}

	private void sendAlreadyMovingMessage() {
		server.sendAlreadyMovingMessage(name);
	}

	private void sendAlreadyRemovingMessage() {
		server.sendAlreadyRemovingMessage(name);
	}

}
